@@ -9,6 +9,14 @@ import (
 	"sync"
 	"time"
 
+	configv1 "github.com/openshift/api/config/v1"
+	"github.com/openshift/client-go/config/clientset/versioned/scheme"
+	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
+	certificatesv1beta1 "k8s.io/client-go/kubernetes/typed/certificates/v1beta1"
+
+	"encoding/base64"
+	"encoding/pem"
+
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -20,10 +28,6 @@ import (
 	"k8s.io/client-go/rest"
 	"k8s.io/klog"
 
-	configv1 "github.com/openshift/api/config/v1"
-	"github.com/openshift/client-go/config/clientset/versioned/scheme"
-	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
-
 	"github.com/openshift/insights-operator/pkg/record"
 )
 
@@ -36,25 +40,29 @@ var (
 	maxEventTimeInterval = 1 * time.Hour
 )
 
+// Gatherer is a driving instance invoking collection of data
 type Gatherer struct {
 	client        configv1client.ConfigV1Interface
 	coreClient    corev1client.CoreV1Interface
 	metricsClient rest.Interface
-
-	lock        sync.Mutex
-	lastVersion *configv1.ClusterVersion
+	certClient    certificatesv1beta1.CertificatesV1beta1Interface
+	lock          sync.Mutex
+	lastVersion   *configv1.ClusterVersion
 }
 
-func New(client configv1client.ConfigV1Interface, coreClient corev1client.CoreV1Interface, metricsClient rest.Interface) *Gatherer {
+// New creates new Gatherer
+func New(client configv1client.ConfigV1Interface, coreClient corev1client.CoreV1Interface, certClient certificatesv1beta1.CertificatesV1beta1Interface, metricsClient rest.Interface) *Gatherer {
 	return &Gatherer{
 		client:        client,
 		coreClient:    coreClient,
+		certClient:    certClient,
 		metricsClient: metricsClient,
 	}
 }
 
 var reInvalidUIDCharacter = regexp.MustCompile(`[^a-z0-9\-]`)
 
+// Gather is hosting and calling all the recording functions
 func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error {
 	return record.Collect(ctx, recorder,
 		func() ([]record.Record, []error) {
@@ -89,6 +97,7 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 			}
 			namespaceEventsCollected := sets.NewString()
 
+			now := time.Now()
 			for _, item := range config.Items {
 				if isHealthyOperator(&item) {
 					continue
@@ -100,7 +109,7 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 						continue
 					}
 					for i := range pods.Items {
-						if isHealthyPod(&pods.Items[i]) {
+						if isHealthyPod(&pods.Items[i], now) {
 							continue
 						}
 						records = append(records, record.Record{Name: fmt.Sprintf("config/pod/%s/%s", pods.Items[i].Namespace, pods.Items[i].Name), Item: PodAnonymizer{&pods.Items[i]}})
@@ -134,6 +143,23 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 
 			return records, nil
 		},
+		func() ([]record.Record, []error) {
+			cms, err := i.coreClient.ConfigMaps("openshift-config").List(metav1.ListOptions{})
+			if err != nil {
+				return nil, []error{err}
+			}
+			records := make([]record.Record, 0, len(cms.Items))
+			for i := range cms.Items {
+				for dk, dv := range cms.Items[i].Data {
+					records = append(records, record.Record{Name: fmt.Sprintf("config/configmaps/%s/%s", cms.Items[i].Name, dk), Item: ConfigMapAnonymizer{v: []byte(dv), encodeBase64: false}})
+				}
+				for dk, dv := range cms.Items[i].BinaryData {
+					records = append(records, record.Record{Name: fmt.Sprintf("config/configmaps/%s/%s", cms.Items[i].Name, dk), Item: ConfigMapAnonymizer{v: dv, encodeBase64: true}})
+				}
+			}
+
+			return records, nil
+		},
 		func() ([]record.Record, []error) {
 			config, err := i.client.ClusterVersions().Get("version", metav1.GetOptions{})
 			if errors.IsNotFound(err) {
@@ -222,6 +248,24 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 			}
 			return []record.Record{{Name: "config/proxy", Item: ProxyAnonymizer{config}}}, nil
 		},
+		func() ([]record.Record, []error) {
+			requests, err := i.certClient.CertificateSigningRequests().List(metav1.ListOptions{})
+			if errors.IsNotFound(err) {
+				return nil, nil
+			}
+			if err != nil {
+				return nil, []error{err}
+			}
+			csrs, err := FromCSRs(requests).Anonymize().Filter(IncludeCSR).Select()
+			if err != nil {
+				return nil, []error{err}
+			}
+			records := make([]record.Record, len(csrs))
+			for i, sr := range csrs {
+				records[i] = record.Record{Name: fmt.Sprintf("config/certificatesigningrequests/%s", sr.ObjectMeta.Name), Item: sr}
+			}
+			return records, nil
+		},
 	)
 }
 
@@ -259,26 +303,34 @@ func (i *Gatherer) gatherNamespaceEvents(namespace string) ([]record.Record, []e
 	return []record.Record{{Name: fmt.Sprintf("events/%s", namespace), Item: EventAnonymizer{&compactedEvents}}}, nil
 }
 
+// RawByte is skipping Marshalling from byte slice
 type RawByte []byte
 
+// Marshal just returns bytes
 func (r RawByte) Marshal(_ context.Context) ([]byte, error) {
 	return r, nil
 }
 
+// Raw is another simplification of marshalling from string
 type Raw struct{ string }
 
+// Marshal returns raw bytes
 func (r Raw) Marshal(_ context.Context) ([]byte, error) {
 	return []byte(r.string), nil
 }
 
+// Anonymizer returns serialized runtime.Object without change
 type Anonymizer struct{ runtime.Object }
 
+// Marshal serializes with OpenShift client-go serializer
 func (a Anonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return runtime.Encode(serializer, a.Object)
 }
 
+// InfrastructureAnonymizer anonymizes infrastructure
 type InfrastructureAnonymizer struct{ *configv1.Infrastructure }
 
+// Marshal serializes Infrastructure with anonymization
 func (a InfrastructureAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return runtime.Encode(serializer, anonymizeInfrastructure(a.Infrastructure))
 }
@@ -291,26 +343,33 @@ func anonymizeInfrastructure(config *configv1.Infrastructure) *configv1.Infrastr
 	return config
 }
 
+// ClusterVersionAnonymizer is serializing ClusterVersion with anonymization
 type ClusterVersionAnonymizer struct{ *configv1.ClusterVersion }
 
+// Marshal serializes ClusterVersion with anonymization
 func (a ClusterVersionAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	a.ClusterVersion.Spec.Upstream = configv1.URL(anonymizeURL(string(a.ClusterVersion.Spec.Upstream)))
 	return runtime.Encode(serializer, a.ClusterVersion)
 }
 
+// FeatureGateAnonymizer implements serializaton of FeatureGate with anonymization
 type FeatureGateAnonymizer struct{ *configv1.FeatureGate }
 
+// Marshal serializes FeatureGate with anonymization
 func (a FeatureGateAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return runtime.Encode(serializer, a.FeatureGate)
 }
 
+// IngressAnonymizer implements serialization with marshalling
 type IngressAnonymizer struct{ *configv1.Ingress }
 
+// Marshal implements serialization of Ingres.Spec.Domain with anonymization
 func (a IngressAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	a.Ingress.Spec.Domain = anonymizeURL(a.Ingress.Spec.Domain)
 	return runtime.Encode(serializer, a.Ingress)
 }
 
+// CompactedEvent holds one Namespace Event
 type CompactedEvent struct {
 	Namespace     string    `json:"namespace"`
 	LastTimestamp time.Time `json:"lastTimestamp"`
@@ -318,17 +377,23 @@ type CompactedEvent struct {
 	Message       string    `json:"message"`
 }
 
+// CompactedEventList is collection of events
 type CompactedEventList struct {
 	Items []CompactedEvent `json:"items"`
 }
+
+// EventAnonymizer implements serializaion of Events with anonymization
 type EventAnonymizer struct{ *CompactedEventList }
 
+// Marshal serializes Events with anonymization
 func (a EventAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return json.Marshal(a.CompactedEventList)
 }
 
+// ProxyAnonymizer implements serialization of HttpProxy/NoProxy with anonymization
 type ProxyAnonymizer struct{ *configv1.Proxy }
 
+// Marshal implements Proxy serialization with anonymization
 func (a ProxyAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	a.Proxy.Spec.HTTPProxy = anonymizeURLCSV(a.Proxy.Spec.HTTPProxy)
 	a.Proxy.Spec.HTTPSProxy = anonymizeURLCSV(a.Proxy.Spec.HTTPSProxy)
@@ -358,8 +423,10 @@ var reURL = regexp.MustCompile(`[^\.\-/\:]`)
 
 func anonymizeURL(s string) string { return reURL.ReplaceAllString(s, "x") }
 
+// ClusterOperatorAnonymizer implements serialization of ClusterOperator without change
 type ClusterOperatorAnonymizer struct{ *configv1.ClusterOperator }
 
+// Marshal serializes ClusterOperator
 func (a ClusterOperatorAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return runtime.Encode(serializer, a.ClusterOperator)
 }
@@ -385,8 +452,10 @@ func namespacesForOperator(operator *configv1.ClusterOperator) []string {
 	return ns
 }
 
+// NodeAnonymizer implements serialization of Node with anonymization
 type NodeAnonymizer struct{ *corev1.Node }
 
+// Marshal implements serialization of Node with anonymization
 func (a NodeAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return runtime.Encode(kubeSerializer, anonymizeNode(a.Node))
 }
@@ -431,8 +500,10 @@ func isHealthyNode(node *corev1.Node) bool {
 	return true
 }
 
+// PodAnonymizer implements serialization with anonymization for a Pod
 type PodAnonymizer struct{ *corev1.Pod }
 
+// Marshal implements serialization of a Pod with anonymization
 func (a PodAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 	return runtime.Encode(kubeSerializer, anonymizePod(a.Pod))
 }
@@ -443,11 +514,13 @@ func anonymizePod(pod *corev1.Pod) *corev1.Pod {
 	return pod
 }
 
-func isHealthyPod(pod *corev1.Pod) bool {
+func isHealthyPod(pod *corev1.Pod, now time.Time) bool {
 	// pending pods may be unable to schedule or start due to failures, and the info they provide in status is important
-	// for identifying why scheduling hass not happened
+	// for identifying why scheduling has not happened
 	if pod.Status.Phase == corev1.PodPending {
-		return false
+		if now.Sub(pod.CreationTimestamp.Time) > 2*time.Minute {
+			return false
+		}
 	}
 	// pods that have containers that have terminated with non-zero exit codes are considered failure
 	for _, status := range pod.Status.InitContainerStatuses {
@@ -484,8 +557,49 @@ func (i *Gatherer) setClusterVersion(version *configv1.ClusterVersion) {
 	i.lastVersion = version.DeepCopy()
 }
 
+// ClusterVersion returns Version for this cluster, which is set by running version during Gathering
 func (i *Gatherer) ClusterVersion() *configv1.ClusterVersion {
 	i.lock.Lock()
 	defer i.lock.Unlock()
 	return i.lastVersion
 }
+
+// ConfigMapAnonymizer implements serialization of configmap
+// and potentially anonymizes if it is a certificate
+type ConfigMapAnonymizer struct {
+	v            []byte
+	encodeBase64 bool
+}
+
+// Marshal implements serialization of Node with anonymization
+func (a ConfigMapAnonymizer) Marshal(_ context.Context) ([]byte, error) {
+	c := []byte(anonymizeConfigMap(a.v))
+	if a.encodeBase64 {
+		buff := make([]byte, base64.StdEncoding.EncodedLen(len(c)))
+		base64.StdEncoding.Encode(buff, []byte(c))
+		c = buff
+	}
+	return c, nil
+}
+
+func anonymizeConfigMap(dv []byte) string {
+	anonymizedPemBlock := `-----BEGIN CERTIFICATE-----
+ANONYMIZED
+-----END CERTIFICATE-----
+`
+	var sb strings.Builder
+	r := dv
+	for {
+		var block *pem.Block
+		block, r = pem.Decode(r)
+		if block == nil {
+			// cannot be extracted
+			return string(dv)
+		}
+		sb.WriteString(anonymizedPemBlock)
+		if len(r) == 0 {
+			break
+		}
+	}
+	return sb.String()
+}
